@@ -40,7 +40,7 @@ MozRepl::RemoteObject - treat Javascript objects as Perl objects
 =cut
 
 use vars qw[$VERSION $objBridge @CARP_NOT @EXPORT_OK $WARN_ON_LEAKS];
-$VERSION = '0.27';
+$VERSION = '0.28';
 
 @EXPORT_OK=qw[as_list];
 @CARP_NOT = (qw[MozRepl::RemoteObject::Instance
@@ -212,7 +212,8 @@ sub to_perl {
     s/"$//;
     
     if (/\.+>/) {
-        warn "Continuation prompt found in [$_]";
+        # This should now be eliminated!
+        die "Continuation prompt found in [$_]";
     }
     
     #warn $js;
@@ -413,8 +414,11 @@ sub install_bridge {
     $options{ queue } ||= [];
     $options{ bufsize } ||= 10_240_000;
     $options{ use_queue } ||= 0; # > 0 means enqueue
-    $options{ max_queue_size } ||= 1000; # mozrepl
-                                         # / Net::Telnet don't like too large commands
+    # mozrepl
+    # / Net::Telnet don't like too large commands
+    $options{ max_queue_size } ||= 1000;
+    
+    $options{ command_sep } ||= "\n--end-remote-input\n";
 
     if (! ref $options{repl}) { # we have host:port
         my @host_port;
@@ -520,14 +524,23 @@ sub install_bridge {
     };
     
     my $rn = $options{repl}->repl;
-    #warn "<$rn>";
     $options{ json } ||= JSON->new->allow_nonref->ascii; # We talk ASCII
+    # Is this still true? It seems to be even when we find an UTF-8 safe
+    # transport above. This needs some investigation.
+    
+    # Switch the Perl-repl to multiline input mode
+    # Well, better use a custom interactor and pass JSON messages that
+    # are self-delimited and contain no newlines. Newline for a new message.
+    
+    # Switch the JS-repl to multiline input mode
+    $options{repl}->execute("$rn.setenv('inputMode','multiline');undefined;\n");
 
     # Load the JS side of the JS <-> Perl bridge
     my $c = $objBridge; # make a copy
     $c =~ s/\[%\s+rn\s+%\]/$rn/g; # cheap templating
     #warn $c;
-    $options{repl}->execute($c);
+    
+    $package->execute_command($c, %options);
     
     $options{ functions } = {}; # cache
     $options{ constants } = {}; # cache
@@ -536,6 +549,16 @@ sub install_bridge {
     bless \%options, $package;    
 };
 
+sub execute_command {
+    my ($self, $command, %options) = @_;
+    $options{ repl } ||= $self->repl;
+    $options{ command_sep } ||= $self->command_sep
+        unless exists $options{ command_sep };
+    $command =~ s/\s+$//;
+    $command .= $options{ command_sep };
+    $options{repl}->execute($command);
+};
+
 =head2 C<< $bridge->expr( $js, $context ) >>
 
 Runs the Javascript passed in through C< $js > and links
@@ -651,7 +674,7 @@ sub queued {
         my $js = join "\n", map { /;$/? $_ : "$_;" } @{ $self->queue };
         # we don't want a result here!
         # This is where we would do ->execute_async on AnyEvent
-        $self->repl->execute($js);
+        $self->execute_command($js);
         @{ $self->queue } = ();
     };
 };
@@ -695,7 +718,7 @@ to be fetched as list, pass C<'list'> as the C<$context>.
 sub declare {
     my ($self,$js,$context) = @_;
     if (! $self->{functions}->{$js}) {
-        $self->{functions}->{$js} = $self->expr($js);
+        $self->{functions}->{$js} = $self->expr("var f=$js;\n;f");
         # Weaken the backlink of the function
         my $res = $self->{functions}->{$js};
         my $ref = ref $res;
@@ -812,12 +835,12 @@ sub js_call_to_perl_struct {
     if (defined wantarray) {
         #warn $js;
         # When going async, we would want to turn this into a callback
-        my $res = $repl->execute($js);
+        my $res = $self->execute_command($js);
         $res =~ s/^(?:\.+\>\s+)+//g;
         while ($res !~ /\S/) {
             # Gobble up continuation prompts
             warn "No result yet from repl";
-            $res = $repl->execute(";\n");
+            $res = $self->execute_command(";"); # no-op
             $res =~ s/^(?:\.+\>\s+)+//g;
         };
         my $d = $self->to_perl($res);
@@ -830,12 +853,13 @@ sub js_call_to_perl_struct {
         #warn "Executing $js";
         # When going async, we would want to turn this into a callback
         # This produces additional, bogus prompts...
-        $repl->execute($js);
+        $self->execute_command($js);
         ()
     };
 };
 
 sub repl {$_[0]->{repl}};
+sub command_sep {$_[0]->{command_sep}};
 sub json {$_[0]->{json}};
 sub name {$_[0]->{repl}?$_[0]->{repl}->repl:undef};
 sub queue {$_[0]->{queue}};
